@@ -18,14 +18,13 @@
 // In case it hasn't been included yet.
 #include "cmaes.hpp"
 
-#include "not_empty_transformation.hpp"
 #include <ensmallen_bits/function.hpp>
 
 namespace ens {
 
 template<typename SelectionPolicyType, typename TransformationPolicyType>
 CMAES<SelectionPolicyType, TransformationPolicyType>::CMAES(const size_t lambda,
-                                  const TransformationPolicyType& 
+                                  const TransformationPolicyType&
                                         transformationPolicy,
                                   const size_t batchSize,
                                   const size_t maxIterations,
@@ -41,35 +40,12 @@ CMAES<SelectionPolicyType, TransformationPolicyType>::CMAES(const size_t lambda,
     stepSize(stepSizeIn)
 { /* Nothing to do. */ }
 
-template<typename SelectionPolicyType, typename TransformationPolicyType>
-CMAES<SelectionPolicyType, TransformationPolicyType>::CMAES(const size_t lambda,
-                                  const double lowerBound,
-                                  const double upperBound,
-                                  const size_t batchSize,
-                                  const size_t maxIterations,
-                                  const double tolerance,
-                                  const SelectionPolicyType& selectionPolicy,
-                                  double stepSizeIn) :
-    lambda(lambda),
-    batchSize(batchSize),
-    maxIterations(maxIterations),
-    tolerance(tolerance),
-    selectionPolicy(selectionPolicy),
-    stepSize(stepSizeIn)
-{
-  Warn << "This is a deprecated constructor and will be removed in a "
-    "future version of ensmallen" << std::endl;
-  NotEmptyTransformation<TransformationPolicyType, EmptyTransformation<>> d;
-  d.Assign(transformationPolicy, lowerBound, upperBound);
-}
-
-
 //! Optimize the function (minimize).
 template<typename SelectionPolicyType, typename TransformationPolicyType>
 template<typename SeparableFunctionType,
          typename MatType,
          typename... CallbackTypes>
-typename MatType::elem_type CMAES<SelectionPolicyType, 
+typename MatType::elem_type CMAES<SelectionPolicyType,
   TransformationPolicyType>::Optimize(
     SeparableFunctionType& function,
     MatType& iterateIn,
@@ -77,7 +53,10 @@ typename MatType::elem_type CMAES<SelectionPolicyType,
 {
   // Convenience typedefs.
   typedef typename MatType::elem_type ElemType;
-  typedef typename MatTypeTraits<MatType>::BaseMatType BaseMatType;
+
+  typedef typename ForwardType<MatType>::bcol bcol;
+  typedef typename ForwardType<MatType>::uvec UVecType;
+  typedef typename ForwardType<MatType>::bmat BaseMatType;
 
   // Make sure that we have the methods that we need.  Long name...
   traits::CheckArbitrarySeparableFunctionTypeAPI<
@@ -95,18 +74,18 @@ typename MatType::elem_type CMAES<SelectionPolicyType,
 
   // Parent weights.
   const size_t mu = std::round(lambda / 2);
-  BaseMatType w = std::log(mu + 0.5) - arma::log(
-      arma::linspace<BaseMatType>(0, mu - 1, mu) + 1.0);
-  w /= arma::accu(w);
+  BaseMatType w = std::log(mu + 0.5) - log(
+      linspace<BaseMatType>(0, mu - 1, mu) + 1.0);
+  w /= accu(w);
 
   // Number of effective solutions.
-  const double muEffective = 1 / arma::accu(arma::pow(w, 2));
+  const double muEffective = 1 / accu(pow(w, 2));
 
   // Step size control parameters.
   BaseMatType sigma(2, 1); // sigma is vector-shaped.
-  if (stepSize == 0) 
+  if (stepSize == 0)
     sigma(0) = transformationPolicy.InitialStepSize();
-  else 
+  else
     sigma(0) = stepSize;
 
   const double cs = (muEffective + 2) / (iterate.n_elem + muEffective + 5);
@@ -151,7 +130,6 @@ typename MatType::elem_type CMAES<SelectionPolicyType,
     terminate |= Callback::Evaluate(*this, function, transformedIterate,
         objective, callbacks...);
   }
-  functionEvaluations += numFunctions;
 
   ElemType overallObjective = currentObjective;
   ElemType lastObjective = std::numeric_limits<ElemType>::max();
@@ -170,13 +148,13 @@ typename MatType::elem_type CMAES<SelectionPolicyType,
   C[0].eye();
 
   // Covariance matrix parameters.
-  arma::Col<ElemType> eigval; // TODO: might need a more general type.
+  bcol eigval; // TODO: might need a more general type.
   BaseMatType eigvec;
   BaseMatType eigvalZero(iterate.n_elem, 1); // eigvalZero is vector-shaped.
   eigvalZero.zeros();
 
   // The current visitation order (sorted by population objectives).
-  arma::uvec idx = arma::linspace<arma::uvec>(0, lambda - 1, lambda);
+  UVecType idx = linspace<UVecType>(0, lambda - 1, lambda);
 
   // Now iterate!
   Callback::BeginOptimization(*this, function, transformedIterate,
@@ -196,22 +174,24 @@ typename MatType::elem_type CMAES<SelectionPolicyType,
     // Perform Cholesky decomposition. If the matrix is not positive definite,
     // add a small value and try again.
     BaseMatType covLower;
-    while (!arma::chol(covLower, C[idx0], "lower"))
+    // while (!chol(covLower, C[idx0], "lower"))
+    while (!chol(covLower, C[idx0]))
       C[idx0].diag() += std::numeric_limits<ElemType>::epsilon();
 
-    arma::eig_sym(eigval, eigvec, C[idx0]);
+    eig_sym(eigval, eigvec, C[idx0]);
 
     for (size_t j = 0; j < lambda; ++j)
     {
       if (iterate.n_rows > iterate.n_cols)
       {
-        pStep[idx(j)] = covLower *
-          arma::randn<BaseMatType>(iterate.n_rows, iterate.n_cols);
+        pStep[idx(j)] = covLower * BaseMatType(
+            iterate.n_rows, iterate.n_cols, GetFillType<MatType>::randn);
       }
       else
       {
-        pStep[idx(j)] = arma::randn<BaseMatType>(iterate.n_rows, iterate.n_cols)
-          * covLower.t();
+        pStep[idx(j)] = BaseMatType(
+          iterate.n_rows, iterate.n_cols, GetFillType<MatType>::randn) *
+          covLower.t();
       }
 
       pPosition[idx(j)] = mPosition[idx0] + sigma(idx0) * pStep[idx(j)];
@@ -223,7 +203,7 @@ typename MatType::elem_type CMAES<SelectionPolicyType,
     }
 
     // Sort population.
-    idx = arma::sort_index(pObjective);
+    idx = sort_index(pObjective);
 
     step = w(0) * pStep[idx(0)];
     for (size_t j = 1; j < mu; ++j)
@@ -236,8 +216,6 @@ typename MatType::elem_type CMAES<SelectionPolicyType,
         transformationPolicy.Transform(mPosition[idx1]), terminate,
         callbacks...);
 
-    functionEvaluations += lambda; 
-
     // Update best parameters.
     if (currentObjective < overallObjective)
     {
@@ -246,30 +224,30 @@ typename MatType::elem_type CMAES<SelectionPolicyType,
 
       transformedIterate = transformationPolicy.Transform(iterate);
       terminate |= Callback::StepTaken(*this, function,
-        transformedIterate, callbacks...);
+          transformedIterate, callbacks...);
     }
 
     // Update Step Size.
     if (iterate.n_rows > iterate.n_cols)
     {
       ps[idx1] = (1 - cs) * ps[idx0] + std::sqrt(
-        cs * (2 - cs) * muEffective) *
-        eigvec * diagmat(1 / eigval) * eigvec.t() * step;
+          cs * (2 - cs) * muEffective) * eigvec *
+          diagmat(1 / eigval) * eigvec.t() * step;
     }
     else
     {
       ps[idx1] = (1 - cs) * ps[idx0] + std::sqrt(
-        cs * (2 - cs) * muEffective) * step *
-        eigvec * diagmat(1 / eigval) * eigvec.t();
+          cs * (2 - cs) * muEffective) * step * eigvec *
+          diagmat(1 / eigval) * eigvec.t();
     }
 
-    const ElemType psNorm = arma::norm(ps[idx1]);
+    const ElemType psNorm = norm(ps[idx1]);
     sigma(idx1) = sigma(idx0) * std::exp(cs / ds * (psNorm / enn - 1));
 
     if (std::isnan(sigma(idx1)) || sigma(idx1) > 1e14)
     {
       Warn << "The step size diverged to " << sigma(idx1) << "; "
-        << "terminating with failure.  Try a smaller step size?" << std::endl;
+          << "terminating with failure.  Try a smaller step size?" << std::endl;
 
       iterate = transformationPolicy.Transform(iterate);
 
@@ -278,20 +256,20 @@ typename MatType::elem_type CMAES<SelectionPolicyType,
     }
 
     // Update covariance matrix.
-    if ((psNorm / sqrt(1 - std::pow(1 - cs, 2 * i))) < h)
+    if ((psNorm / std::sqrt(1 - std::pow(1.0 - cs, 2.0 * (double) i))) < h)
     {
       pc[idx1] = (1 - cc) * pc[idx0] + std::sqrt(cc * (2 - cc) *
-        muEffective) * step;
+          muEffective) * step;
 
       if (iterate.n_rows > iterate.n_cols)
       {
         C[idx1] = (1 - c1 - cmu) * C[idx0] + c1 *
-          (pc[idx1] * pc[idx1].t());
+            (pc[idx1] * pc[idx1].t());
       }
       else
       {
         C[idx1] = (1 - c1 - cmu) * C[idx0] + c1 *
-          (pc[idx1].t() * pc[idx1]);
+            (pc[idx1].t() * pc[idx1]);
       }
     }
     else
@@ -301,12 +279,12 @@ typename MatType::elem_type CMAES<SelectionPolicyType,
       if (iterate.n_rows > iterate.n_cols)
       {
         C[idx1] = (1 - c1 - cmu) * C[idx0] + c1 * (pc[idx1] *
-          pc[idx1].t() + (cc * (2 - cc)) * C[idx0]);
+            pc[idx1].t() + (cc * (2 - cc)) * C[idx0]);
       }
       else
       {
         C[idx1] = (1 - c1 - cmu) * C[idx0] + c1 *
-          (pc[idx1].t() * pc[idx1] + (cc * (2 - cc)) * C[idx0]);
+            (pc[idx1].t() * pc[idx1] + (cc * (2 - cc)) * C[idx0]);
       }
     }
 
@@ -314,21 +292,19 @@ typename MatType::elem_type CMAES<SelectionPolicyType,
     {
       for (size_t j = 0; j < mu; ++j)
       {
-        C[idx1] = C[idx1] + cmu * w(j) *
-          pStep[idx(j)] * pStep[idx(j)].t();
+        C[idx1] = C[idx1] + cmu * w(j) * pStep[idx(j)] * pStep[idx(j)].t();
       }
     }
     else
     {
       for (size_t j = 0; j < mu; ++j)
       {
-        C[idx1] = C[idx1] + cmu * w(j) *
-          pStep[idx(j)].t() * pStep[idx(j)];
+        C[idx1] = C[idx1] + cmu * w(j) * pStep[idx(j)].t() * pStep[idx(j)];
       }
     }
 
-    arma::eig_sym(eigval, eigvec, C[idx1]);
-    const arma::uvec negativeEigval = arma::find(eigval < 0, 1);
+    eig_sym(eigval, eigvec, C[idx1]);
+    const UVecType negativeEigval = find(eigval < 0, 1);
     if (!negativeEigval.is_empty())
     {
       if (negativeEigval(0) == 0)
@@ -338,19 +314,19 @@ typename MatType::elem_type CMAES<SelectionPolicyType,
       else
       {
         C[idx1] = eigvec.cols(0, negativeEigval(0) - 1) *
-          arma::diagmat(eigval.subvec(0, negativeEigval(0) - 1)) *
-          eigvec.cols(0, negativeEigval(0) - 1).t();
+            diagmat(eigval.subvec(0, negativeEigval(0) - 1)) *
+            eigvec.cols(0, negativeEigval(0) - 1).t();
       }
     }
 
     // Output current objective function.
     Info << "CMA-ES: iteration " << i << ", objective " << overallObjective
-      << "." << std::endl;
+        << "." << std::endl;
 
     if (std::isnan(overallObjective) || std::isinf(overallObjective))
     {
       Warn << "CMA-ES: converged to " << overallObjective << "; "
-        << "terminating with failure.  Try a smaller step size?" << std::endl;
+          << "terminating with failure.  Try a smaller step size?" << std::endl;
 
       iterate = transformationPolicy.Transform(iterate);
       Callback::EndOptimization(*this, function, iterate, callbacks...);
@@ -361,7 +337,7 @@ typename MatType::elem_type CMAES<SelectionPolicyType,
     {
       if (steps > patience) {
         Info << "CMA-ES: minimized within tolerance " << tolerance << "; "
-          << "terminating optimization." << std::endl;
+            << "terminating optimization." << std::endl;
 
         iterate = transformationPolicy.Transform(iterate);
         Callback::EndOptimization(*this, function, iterate, callbacks...);
