@@ -37,7 +37,9 @@ class SchafferFunctionN1
   size_t numVariables;
 
  public:
- //! Initialize the SchafferFunctionN1
+  typedef typename MatType::elem_type ElemType;
+
+  // Initialize the SchafferFunctionN1 object.
   SchafferFunctionN1() : numObjectives(2), numVariables(1)
   {/* Nothing to do here. */}
 
@@ -54,8 +56,8 @@ class SchafferFunctionN1
 
     arma::Col<ElemType> objectives(numObjectives);
 
-    objectives(0) = std::pow(coords[0], 2);
-    objectives(1) = std::pow(coords[0] - 2, 2);
+    objectives(0) = std::pow(coords[0], ElemType(2));
+    objectives(1) = std::pow(coords[0] - 2, ElemType(2));
 
     return objectives;
   }
@@ -71,17 +73,17 @@ class SchafferFunctionN1
 
   struct ObjectiveA
   {
-    typename MatType::elem_type Evaluate(const MatType& coords)
+    ElemType Evaluate(const MatType& coords)
     {
-        return std::pow(coords[0], 2);
+      return std::pow(coords[0], ElemType(2));
     }
   } objectiveA;
 
   struct ObjectiveB
   {
-    typename MatType::elem_type Evaluate(const MatType& coords)
+    ElemType Evaluate(const MatType& coords)
     {
-        return std::pow(coords[0] - 2, 2);
+      return std::pow(coords[0] - 2, ElemType(2));
     }
   } objectiveB;
 
@@ -91,7 +93,8 @@ class SchafferFunctionN1
     return std::make_tuple(objectiveA, objectiveB);
   }
 };
+
 } // namespace test
 } // namespace ens
 
-#endif
\ No newline at end of file
+#endif
