@@ -12,6 +12,7 @@
 
 #include <ensmallen.hpp>
 #include "catch.hpp"
+#include "test_types.hpp"
 
 using namespace arma;
 using namespace ens;
@@ -56,7 +57,8 @@ void CreateLovaszThetaInitialPoint(const MatType& edges,
  */
 template<typename MatType>
 void SetupLovaszTheta(const MatType& edges,
-                      LRSDP<SDP<MatType>>& lovasz)
+                      LRSDP<SDP<MatType>>& lovasz,
+                      typename ForwardType<MatType>::bvec& lambda)
 {
   // Get the number of vertices in the problem.
   const size_t vertices = max(max(edges)) + 1;
@@ -81,21 +83,23 @@ void SetupLovaszTheta(const MatType& edges,
   }
 
   // Set the Lagrange multipliers right.
-  lovasz.AugLag().Lambda().ones(edges.n_cols + 1);
-  lovasz.AugLag().Lambda() *= -1;
-  lovasz.AugLag().Lambda()[0] = -((typename MatType::elem_type) vertices);
+  lambda.ones(edges.n_cols + 1);
+  lambda *= -1;
+  lambda[0] = -((typename MatType::elem_type) vertices);
 }
 
 /**
  * johnson8-4-4.co test case for Lovasz-Theta LRSDP.
  * See Monteiro and Burer 2004.
  */
-TEST_CASE("Johnson844LovaszThetaSDP", "[LRSDPTest]")
+TEMPLATE_TEST_CASE("Johnson844LovaszThetaSDP", "[LRSDP]", ENS_TEST_TYPES)
 {
-  // Load the edges.
-  arma::mat edges;
+  typedef typename TestType::elem_type ElemType;
 
-  if (edges.load("data/johnson8-4-4.csv", arma::csv_ascii) == false)
+  // Load the edges.
+  TestType edges;
+
+  if (edges.load("data/johnson8-4-4.csv", csv_ascii) == false)
   {
     FAIL("couldn't load data");
     return;
@@ -104,80 +108,42 @@ TEST_CASE("Johnson844LovaszThetaSDP", "[LRSDPTest]")
   edges = edges.t();
 
   // The LRSDP itself and the initial point.
-  arma::mat coordinates;
+  TestType coordinates;
 
   CreateLovaszThetaInitialPoint(edges, coordinates);
 
-  LRSDP<SDP<arma::mat>> lovasz(edges.n_cols + 1, 0, coordinates);
+  LRSDP<SDP<TestType>> lovasz(edges.n_cols + 1, 0, coordinates);
 
-  SetupLovaszTheta(edges, lovasz);
+  typedef typename ForwardType<TestType>::bvec VecType;
+  VecType lambda;
+  SetupLovaszTheta(edges, lovasz, lambda);
 
-  double finalValue = lovasz.Optimize(coordinates);
+  double sigma = 10;
+  ElemType finalValue = lovasz.Optimize(coordinates, lambda, sigma);
 
   // Final value taken from Monteiro + Burer 2004.
-  REQUIRE(finalValue == Approx(-14.0).epsilon(1e-7));
+  REQUIRE(finalValue == Approx(-14.0).epsilon(Tolerances<TestType>::Obj));
 
   // Now ensure that all the constraints are satisfied.
-  arma::mat rrt = coordinates * trans(coordinates);
-  REQUIRE(trace(rrt) == Approx(1.0).epsilon(1e-7));
+  TestType rrt = coordinates * trans(coordinates);
+  REQUIRE(trace(rrt) == Approx(1.0).epsilon(Tolerances<TestType>::Obj));
 
   // All those edge constraints...
   for (size_t i = 0; i < edges.n_cols; ++i)
   {
-    REQUIRE(rrt(edges(0, i), edges(1, i)) == Approx(0.0).margin(1e-5));
-    REQUIRE(rrt(edges(1, i), edges(0, i)) == Approx(0.0).margin(1e-5));
+    REQUIRE(rrt(edges(0, i), edges(1, i)) ==
+        Approx(0.0).margin(100 * Tolerances<TestType>::Obj));
+    REQUIRE(rrt(edges(1, i), edges(0, i)) ==
+        Approx(0.0).margin(100 * Tolerances<TestType>::Obj));
   }
 }
 
-/**
- * johnson8-4-4.co test case for Lovasz-Theta LRSDP.
- * See Monteiro and Burer 2004.  Uses arma::fmat.
- */
-TEST_CASE("Johnson844LovaszThetaFMatSDP", "[LRSDPTest]")
-{
-  // Load the edges.
-  arma::fmat edges;
-
-  if (edges.load("data/johnson8-4-4.csv", arma::csv_ascii) == false)
-  {
-    FAIL("couldn't load data");
-    return;
-  }
-
-  edges = edges.t();
-
-  // The LRSDP itself and the initial point.
-  arma::fmat coordinates;
-
-  CreateLovaszThetaInitialPoint(edges, coordinates);
-
-  LRSDP<SDP<arma::fmat>> lovasz(edges.n_cols + 1, 0, coordinates);
-
-  SetupLovaszTheta(edges, lovasz);
-
-  float finalValue = lovasz.Optimize(coordinates);
-
-  // Final value taken from Monteiro + Burer 2004.
-  REQUIRE(finalValue == Approx(-14.0).epsilon(0.1));
-
-  // Now ensure that all the constraints are satisfied.
-  arma::fmat rrt = coordinates * trans(coordinates);
-  REQUIRE(trace(rrt) == Approx(1.0).epsilon(0.1));
-
-  // All those edge constraints...
-  for (size_t i = 0; i < edges.n_cols; ++i)
-  {
-    REQUIRE(rrt(edges(0, i), edges(1, i)) == Approx(0.0).margin(0.1));
-    REQUIRE(rrt(edges(1, i), edges(0, i)) == Approx(0.0).margin(0.1));
-  }
-}
-
-
 /**
  * Create an unweighted graph laplacian from the edges.
  */
-void CreateSparseGraphLaplacian(const arma::mat& edges,
-                                arma::sp_mat& laplacian)
+template<typename ElemType>
+void CreateSparseGraphLaplacian(const Mat<ElemType>& edges,
+                                SpMat<ElemType>& laplacian)
 {
   // Get the number of vertices in the problem.
   const size_t vertices = max(max(edges)) + 1;
@@ -186,22 +152,24 @@ void CreateSparseGraphLaplacian(const arma::mat& edges,
 
   for (size_t i = 0; i < edges.n_cols; ++i)
   {
-    laplacian(edges(0, i), edges(1, i)) = -1.0;
-    laplacian(edges(1, i), edges(0, i)) = -1.0;
+    laplacian(edges(0, i), edges(1, i)) = ElemType(-1);
+    laplacian(edges(1, i), edges(0, i)) = ElemType(-1);
   }
 
   for (size_t i = 0; i < vertices; ++i)
   {
-    laplacian(i, i) = -arma::accu(laplacian.row(i));
+    laplacian(i, i) = -accu(laplacian.row(i));
   }
 }
 
-TEST_CASE("ErdosRenyiRandomGraphMaxCutSDP", "[LRSDPTest]")
+TEMPLATE_TEST_CASE("ErdosRenyiRandomGraphMaxCutSDP", "[LRSDP]", ENS_TEST_TYPES)
 {
-  // Load the edges.
-  arma::mat edges;
+  typedef typename TestType::elem_type ElemType;
 
-  if (edges.load("data/erdosrenyi-n100.csv", arma::csv_ascii) == false)
+  // Load the edges.
+  TestType edges;
+
+  if (edges.load("data/erdosrenyi-n100.csv", csv_ascii) == false)
   {
     FAIL("couldn't load data");
     return;
@@ -209,7 +177,7 @@ TEST_CASE("ErdosRenyiRandomGraphMaxCutSDP", "[LRSDPTest]")
 
   edges = edges.t();
 
-  arma::sp_mat laplacian;
+  SpMat<ElemType> laplacian;
   CreateSparseGraphLaplacian(edges, laplacian);
 
   float r = 0.5 + sqrt(0.25 + 2 * edges.n_cols);
@@ -217,33 +185,34 @@ TEST_CASE("ErdosRenyiRandomGraphMaxCutSDP", "[LRSDPTest]")
     r = laplacian.n_rows;
 
   // initialize coordinates to a feasible point
-  arma::mat coordinates(laplacian.n_rows, ceil(r));
+  TestType coordinates(laplacian.n_rows, ceil(r));
   coordinates.zeros();
   for (size_t i = 0; i < coordinates.n_rows; ++i)
   {
-    coordinates(i, i % coordinates.n_cols) = 1.;
+    coordinates(i, i % coordinates.n_cols) = ElemType(1);
   }
 
-  LRSDP<SDP<arma::sp_mat>> maxcut(laplacian.n_rows, 0, coordinates);
+  LRSDP<SDP<SpMat<ElemType>>> maxcut(laplacian.n_rows, 0, coordinates);
   maxcut.SDP().C() = laplacian;
   maxcut.SDP().C() *= -1.; // need to minimize the negative
   maxcut.SDP().SparseB().ones(laplacian.n_rows);
   for (size_t i = 0; i < laplacian.n_rows; ++i)
   {
     maxcut.SDP().SparseA()[i].zeros(laplacian.n_rows, laplacian.n_rows);
-    maxcut.SDP().SparseA()[i](i, i) = 1.;
+    maxcut.SDP().SparseA()[i](i, i) = ElemType(1);
   }
 
-  const double finalValue = maxcut.Optimize(coordinates);
-  const arma::mat rrt = coordinates * trans(coordinates);
+  const ElemType finalValue = maxcut.Optimize(coordinates);
+  const TestType rrt = coordinates * trans(coordinates);
 
   for (size_t i = 0; i < laplacian.n_rows; ++i)
   {
-    REQUIRE(rrt(i, i) == Approx(1.0).epsilon(1e-7));
+    REQUIRE(rrt(i, i) == Approx(1.0).epsilon(Tolerances<TestType>::Obj));
   }
 
   // Final value taken by solving with Mosek
-  REQUIRE(finalValue == Approx(-3672.7).epsilon(1e-3));
+  REQUIRE(finalValue ==
+      Approx(-3672.7).epsilon(100 * Tolerances<TestType>::Coord));
 }
 
 /*
@@ -273,18 +242,20 @@ TEST_CASE("ErdosRenyiRandomGraphMaxCutSDP", "[LRSDPTest]")
  *    SIAM Review 2010.
  *
  */
-TEST_CASE("GaussianMatrixSensingSDP", "[LRSDPTest]")
+TEMPLATE_TEST_CASE("GaussianMatrixSensingSDP", "[LRSDP]", ENS_TEST_TYPES)
 {
-  arma::mat Xorig, A;
+  typedef typename TestType::elem_type ElemType;
+
+  TestType Xorig, A;
 
   // read the unknown matrix X and the measurement matrices A_i in
 
-  if (Xorig.load("data/sensing_X.csv", arma::csv_ascii) == false)
+  if (Xorig.load("data/sensing_X.csv", csv_ascii) == false)
   {
     FAIL("couldn't load data");
   }
 
-  if( A.load("data/sensing_A.csv", arma::csv_ascii) == false)
+  if (A.load("data/sensing_A.csv", csv_ascii) == false)
   {
     FAIL("couldn't load data");
   }
@@ -294,49 +265,51 @@ TEST_CASE("GaussianMatrixSensingSDP", "[LRSDPTest]")
   const size_t p = A.n_rows;
   assert(A.n_cols == m * m);
 
-  arma::vec b(p);
+  Col<ElemType> b(p);
   for (size_t i = 0; i < p; ++i)
   {
-    const arma::mat Ai = arma::reshape(A.row(i), n, m);
-    b(i) = arma::dot(trans(Ai), Xorig);
+    const TestType Ai = reshape(A.row(i), n, m);
+    b(i) = dot(trans(Ai), Xorig);
   }
 
   float r = 0.5 + sqrt(0.25 + 2 * p);
   if (ceil(r) > m + n)
     r = m + n;
 
-  arma::mat coordinates;
+  TestType coordinates;
   coordinates.eye(m + n, ceil(r));
 
-  LRSDP<SDP<arma::sp_mat>> sensing(0, p, coordinates, 15);
+  LRSDP<SDP<SpMat<ElemType>>> sensing(0, p, coordinates, 15);
   sensing.SDP().C().eye(m + n, m + n);
   sensing.SDP().DenseB() = 2. * b;
 
-  const auto blockRows = arma::span(0, m - 1);
-  const auto blockCols = arma::span(m, m + n - 1);
+  const span blockRows(0, m - 1);
+  const span blockCols(m, m + n - 1);
 
   for (size_t i = 0; i < p; ++i)
   {
-    const arma::mat Ai = arma::reshape(A.row(i), n, m);
+    const TestType Ai = reshape(A.row(i), n, m);
     sensing.SDP().DenseA()[i].zeros(m + n, m + n);
     sensing.SDP().DenseA()[i](blockRows, blockCols) = trans(Ai);
     sensing.SDP().DenseA()[i](blockCols, blockRows) = Ai;
   }
 
-  double finalValue = sensing.Optimize(coordinates);
-  REQUIRE(finalValue == Approx(44.7550132629).epsilon(1e-3));
+  ElemType finalValue = sensing.Optimize(coordinates);
+  REQUIRE(finalValue == Approx(44.7550132629).epsilon(
+      Tolerances<TestType>::LargeObj));
 
-  const arma::mat rrt = coordinates * trans(coordinates);
+  const TestType rrt = coordinates * trans(coordinates);
   for (size_t i = 0; i < p; ++i)
   {
-    const arma::mat Ai = arma::reshape(A.row(i), n, m);
-    const double measurement =
-        arma::dot(trans(Ai), rrt(blockRows, blockCols));
-    REQUIRE(measurement == Approx(b(i)).epsilon(0.0006));
+    const TestType Ai = reshape(A.row(i), n, m);
+    const ElemType measurement = dot(trans(Ai), rrt(blockRows, blockCols));
+    // Custom tolerances because floats can do very bad here.
+    const ElemType eps = std::is_same<ElemType, float>::value ? 0.1 : 0.001;
+    REQUIRE(measurement == Approx(b(i)).epsilon(eps));
   }
 
   // check matrix recovery
-  const double err = arma::norm(Xorig - rrt(blockRows, blockCols), "fro") /
-      arma::norm(Xorig, "fro");
-  REQUIRE(err == Approx(0.0).margin(0.05));
+  const ElemType err = norm(Xorig - rrt(blockRows, blockCols), "fro") /
+      norm(Xorig, "fro");
+  REQUIRE(err == Approx(0.0).margin(10 * Tolerances<TestType>::LargeObj));
 }
