@@ -9,6 +9,10 @@
  */
 
 #include <iostream>
+#if defined(ENS_USE_COOT)
+  #include <armadillo>
+  #include <bandicoot>
+#endif
 #include <ensmallen.hpp>
 
 //#define CATCH_CONFIG_MAIN  // catch.hpp will define main()
@@ -17,6 +21,10 @@
 
 int main(int argc, char** argv)
 {
+  #ifdef ENS_HAVE_COOT
+  coot::get_rt().init(true);
+  #endif
+
   Catch::Session session;
   const int returnCode = session.applyCommandLine(argc, argv);
   // Check for a command line error.
@@ -26,6 +34,10 @@ int main(int argc, char** argv)
   std::cout << "ensmallen version: " << ens::version::as_string() << std::endl;
   std::cout << "armadillo version: " << arma::arma_version::as_string() << std::endl;
 
+  #ifdef ENS_HAVE_COOT
+  std::cout << "bandicoot version: " << coot::coot_version::as_string() << std::endl;
+  #endif
+
   // Use Catch2 command-line to set the random seed.
   // -rng-seed <'time'|number>
   // If a number is provided this is used directly as the seed. Alternatively
